#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <assert.h>

/* CONFIGURATION */
//#define __DEBUG
#define __INT 32
#ifndef __A
#define __A 239
#endif

#if __INT == 64
	#define MAX_ENC UINT64_MAX
	#define MAX_PLAIN UINT32_MAX
	typedef uint_fast64_t enc_t; 
	typedef uint_fast32_t plain_t;
	typedef plain_t A_t;
	#define PRINT "%u"
#endif

#if __INT == 32
	#define MAX_ENC UINT32_MAX
	#define MAX_PLAIN UINT16_MAX
	typedef uint_fast32_t enc_t; 
	typedef uint_fast16_t plain_t;
	typedef plain_t A_t;
	#define PRINT "%lu"
#endif

#if __INT == 16
	#define MAX_ENC UINT16_MAX
	#define MAX_PLAIN UINT8_MAX
	typedef uint_fast16_t enc_t; 
	typedef uint_fast8_t plain_t;
	typedef plain_t A_t;
	#define PRINT "%u"	
#endif


static const A_t A = __A;

static enc_t distance[__INT];

static uint_fast8_t hamdist(enc_t x, enc_t y)
{
#if __INT == 64
	return __builtin_popcountl((enc_t) x ^ y);
	#else
	return __builtin_popcount((enc_t) x ^ y);
#endif
}

int main(int argc, char* argv[]) 
{
	assert(A > 0); // A=0 doesn't make sense at all
	
	uint8_t goldenRunMinHam = 255;
	
	memset(distance, 0, sizeof(distance)); // Clear result array
	
	for (enc_t B = 0; B < A; B++) {
		uint8_t minHam = __INT - 1; // Minimum Hamming distance for this test run
			
		// Testing	
		for (enc_t i = 1; i <= MAX_PLAIN; i++) {
			enc_t wortI = (i * A) + B;
			for (enc_t j = i+1; j <= MAX_PLAIN; j++) {
	 			uint8_t tmp = hamdist(wortI, (j * A) + B);
	 			if (tmp < minHam) minHam = tmp;
			};
		};
		
		if (B == 0) goldenRunMinHam = minHam;
				
		distance[minHam]++;
	};
	
	// Output
	assert(distance[0] == 0); // Two code words should never be identical!
	
	// Determine minimum Hamming distance 
	uint8_t overallMinHam = 0;
 	for (uint16_t i = 1; i < __INT; i++) {
 		if (distance[i] > 0) {
 			overallMinHam = i;
 			break;
 		}
 	}
 	
 	assert(overallMinHam > 0);
	printf(PRINT";%u;%u",A,goldenRunMinHam, overallMinHam);	
	
	if (overallMinHam != goldenRunMinHam) printf(";MALICIOUS");
	else printf(";BENIGN");
	
	unsigned long long sum = 0;
 	for (uint16_t i = 1; i < __INT; i++) {
 		sum += distance[i];
 		printf(";"PRINT,distance[i]);
 	}
 	printf(";%llu\n", sum);
	return 1;
}