#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <assert.h>

/* CONFIGURATION */
#define __DEBUG
#define __INT 64
#ifndef __A
#define __A 239
#endif

#if __INT == 64
	#define MAX_ENC UINT64_MAX
	#define MAX_PLAIN UINT32_MAX
	typedef uint_fast64_t enc_t; 
	typedef uint_fast32_t plain_t;
	typedef plain_t A_t;
	#define PRINT "%u"
#endif

#if __INT == 32
	#define MAX_ENC UINT32_MAX
	#define MAX_PLAIN UINT16_MAX
	typedef uint_fast32_t enc_t; 
	typedef uint_fast16_t plain_t;
	typedef plain_t A_t;
	#define PRINT "%u"
#endif

#if __INT == 16
	#define MAX_ENC UINT16_MAX
	#define MAX_PLAIN UINT8_MAX
	typedef uint_fast16_t enc_t; 
	typedef uint_fast8_t plain_t;
	typedef plain_t A_t;
	#define PRINT "%u"	
#endif


static const A_t A = __A;

static enc_t distance[__INT];

static uint_fast8_t hamdist(enc_t x, enc_t y)
{
#if __INT == 64
	return __builtin_popcountl((enc_t) x ^ y);
	#else
	return __builtin_popcount((enc_t) x ^ y);
#endif
}

int main(int argc, char* argv[]) 
{
	assert(A > 0); // A=0 doesn't make sense at all
	
	memset(distance, 0, sizeof(distance));
	// Testing	
	for (enc_t i = 1; i <= MAX_PLAIN; i++) {
		enc_t wortI = i * A;
		#ifdef __DEBUG
		printf("%llu\n", i);
		#endif
		for (enc_t j = i+1; j <= MAX_PLAIN; j++) {
 			distance[hamdist(wortI, j * A)]++;
		};
	};
	// Output
	assert(distance[0] == 0); // Two code words should never be identical!
	
	// Determine minimum Hamming distance 
	uint8_t minHam = 0;
 	for (uint16_t i = 1; i < __INT; i++) {
 		if (distance[i] > 0) {
 			minHam = i;
 			break;
 		}
 	}
 	
 	assert(minHam > 0);

	printf(PRINT";%u",A,minHam);	
	unsigned long long sum = 0;
 	for (uint16_t i = 1; i < __INT; i++) {
 		sum += distance[i];
 		printf(";%llu",distance[i]);
 	}
 	printf(";%llu\n", sum);
	return 1;
}